// Fill out your copyright notice in the Description page of Project Settings.


#include "Controllers/DDMorphShapingController.h"
#include "DataAssets/DDDataAssets.h"
#include "ActorComponents/DDEditableSkeletalMeshComponent.h"


void UDDMorphShapingController::Initialize(UDDCharacterEditorConfigData* Config)
{
    Configuration = Config;
}

void UDDMorphShapingController::SetModel(UDDMorphShapingModel* Model)
{
	ModelToControl = Model;

}

void UDDMorphShapingController::ResetToConfigValues()
{

}

void UDDMorphShapingController::Randomize()
{

}

// Compute barycentric coordinates (u, v, w) for
// point p with respect to triangle (a, b, c)
void Barycentric(const FVector2f p,
    const FVector2f& a,
    const FVector2f& b,
    const FVector2f& c,
    float* u, float* v, float* w)
{
    FVector2f v0 = b - a, v1 = c - a, v2 = p - a;
    float d00 = FVector2f::DotProduct(v0, v0);
    float d01 = FVector2f::DotProduct(v0, v1);
    float d11 = FVector2f::DotProduct(v1, v1);
    float d20 = FVector2f::DotProduct(v2, v0);
    float d21 = FVector2f::DotProduct(v2, v1);
    float denom = d00 * d11 - d01 * d01;
    *v = (d11 * d20 - d01 * d21) / denom;
    *w = (d00 * d21 - d01 * d20) / denom;
    *u = 1.0f - *v - *w;
}

void UDDMorphShapingController::UpdateModel(
    const FDDShapeConstraints* ShapeConstraints,
    int i0, float v0, int i1, float v1)
{
    ModelToControl->MarkedDirty = true;

    for (int32 i = 0; i < ShapeConstraints->shapes.Num(); i++)
    {
        ModelToControl->Weights[ShapeConstraints->shapes[i]] = 0.0f;
    }

    ModelToControl->Weights[ShapeConstraints->shapes[i0]] = v0;
    if (i1 >= 0)
    {
        ModelToControl->Weights[ShapeConstraints->shapes[i1]] = v1;
    }
}

void UDDMorphShapingController::ApplyChanges(UDDEditableSkeletalMeshComponent* SkeletalMeshComponent)
{
    ModelToControl->MarkedDirty = false;
    for (const TPair<FString, float>& Pair : ModelToControl->Weights)
    {
        SkeletalMeshComponent->SetMorphTarget(*Pair.Key, Pair.Value);
    }
}

