#include "Controllers/ImageProcessor/DDImageProcessor.h"
#include "RHICommandList.h"
#include "RHIStaticStates.h"
#include "Engine/Texture2D.h"
#include "Engine/TextureRenderTarget2D.h"
#include "RenderGraph.h"
#include "RenderGraphUtils.h"
#include "RenderUtils.h"
#include "GlobalShader.h"
#include "TextureResource.h"


void DDImageProcessorExecuteContext_GameThread::Execute(UDDImageProcessor* ip)
{
	ip->Execute(*this);
    ip->Executed_GameThread = true;
}

void DDImageProcessorExecuteContext_RenderThread::Execute(UDDImageProcessor* ip)
{
	ip->Execute(*this);
    ip->Executed_RenderThread = true;
}

void UDDependantImageProcessor::SetDependency(UDDImageProcessor* Dependency, int32 Slot)
{
    FScopeLock Lock(&DataMutex);
    SlotToDependency.Add(Slot, Dependency);
}

void UDDependantImageProcessor::Prepare()
{
    UDDImageProcessor::Prepare();

    for (const TPair<int32, UDDImageProcessor*>& Pair : SlotToDependency)
    {
        UDDImageProcessor* Dependency = Pair.Value;
        Dependency->Prepare();
    }
}

void UDDependantImageProcessor::ExecuteDependencies(DDImageProcessorExecuteContext_GameThread& context)
{
    context.Depth += 1;
    for (const TPair<int32, UDDImageProcessor*>& Pair : SlotToDependency)
    {
        UDDImageProcessor* Dependency = Pair.Value;
        if (Dependency->Executed_GameThread)
            continue;
        context.Execute(Dependency);
    }
    context.Depth -= 1;
}

void UDDependantImageProcessor::ExecuteDependencies(DDImageProcessorExecuteContext_RenderThread& context)
{
    context.Depth += 1;
    for (const TPair<int32, UDDImageProcessor*>& Pair : SlotToDependency)
    {
        UDDImageProcessor* Dependency = Pair.Value;
        if (Dependency->Executed_RenderThread)
            continue;
        context.Execute(Dependency);
    }
    context.Depth -= 1;
}

FRDGTextureSRV* UDDependantImageProcessor::InputDependencyAsSRV(DDImageProcessorExecuteContext_RenderThread& context, int32 DependencyIndex)
{
    auto Input = SlotToDependency[DependencyIndex]->GetResultRenderTarget()->GetRenderTargetResource();
    FRDGTextureRef InputRef = RegisterExternalTexture(*context.GraphBuilder, Input->GetRenderTargetTexture(), TEXT("Popul8_shader_CS.Input"));
    FRDGTextureSRV* inputSRV = context.GraphBuilder->CreateSRV(InputRef);
    return inputSRV;
}

void UDDImageProcessor::CreateResultRenderTarget(const FLinearColor& clearColor, int32 sizeX, int32 sizeY, ETextureRenderTargetFormat format)
{
    ResultRenderTarget = NewObject<UTextureRenderTarget2D>(this);
    ResultRenderTarget->RenderTargetFormat = format;
    ResultRenderTarget->ClearColor = clearColor;
    ResultRenderTarget->InitAutoFormat(sizeX, sizeY);
    ResultRenderTarget->UpdateResourceImmediate(true);
}

void UDDImageProcessor::Initialize(int32 Width, int32 Height)
{
    CreateResultRenderTarget(FLinearColor::Black, Width, Height);
}

void UDDImageProcessor::Prepare()
{
    check(ResultRenderTarget);
    Executed_GameThread = false;
    Executed_RenderThread = false;
}
