// Fill out your copyright notice in the Description page of Project Settings.

#include "DataAssets/DDCharacterEditorSkinCompositionData.h"
#include "JsonObjectConverter.h"
#include "IImageWrapperModule.h"
#include "IImageWrapper.h"
#include "ImageUtils.h"
#include "Engine/Texture2D.h"
#include "Misc/Paths.h"
#include "Misc/FileHelper.h"
#include "Modules/ModuleManager.h"
#include "UObject/SoftObjectPtr.h"


TStrongObjectPtr<UDDCharacterEditorSkinCompositionData> UDDCharacterEditorSkinCompositionData::Load(const FString& Path)
{
    FString DirectoryPath = FPaths::GetPath(Path);
    FString jsonString;
    if (!FFileHelper::LoadFileToString(jsonString, *Path))
    {
        UE_LOG(LogTemp, Warning, TEXT("Failed to load file: %s"), *Path);
        return nullptr;
    }

    // Parse the JSON string
    TSharedPtr<FJsonObject> JsonObject;
    TSharedRef<TJsonReader<>> jsonReader = TJsonReaderFactory<>::Create(jsonString);
    bool result = FJsonSerializer::Deserialize(jsonReader, JsonObject);
    check(result);
    result = JsonObject.IsValid();
    check(result);

    // Create a new instance
    TStrongObjectPtr<UDDCharacterEditorSkinCompositionData> NewAsset = 
        TStrongObjectPtr<UDDCharacterEditorSkinCompositionData>(NewObject<UDDCharacterEditorSkinCompositionData>());
    if (!NewAsset)
    {
        UE_LOG(LogTemp, Error, TEXT("Failed to create UDDCharacterEditorSkinCompositionData object."));
        return nullptr;
    }

    // Populate the asset from JSON
    if (!FJsonObjectConverter::JsonObjectToUStruct(JsonObject.ToSharedRef(), 
        UDDCharacterEditorSkinCompositionData::StaticClass(), NewAsset.Get(), 0, 0))
    {
        UE_LOG(LogTemp, Warning, TEXT("Failed to parse JSON into UDDCharacterEditorSkinCompositionData"));
        return nullptr;
    }

    return NewAsset;
}