// Fill out your copyright notice in the Description page of Project Settings.


#include "Views/DDCharacterEditorExpandableArea.h"
#include "Blueprint/WidgetLayoutLibrary.h"
#include "Components/TextBlock.h"
#include "Components/VerticalBox.h"
#include "Components/VerticalBoxSlot.h"
#include "Components/HorizontalBox.h"
#include "Components/HorizontalBoxSlot.h"
#include "Components/Slider.h"
#include "DataAssets/DDDataAssets.h"
#include "DataAssets/DDCharacterEditorSkinCompositionData.h"


void UDDCharacterEditorExpandableArea::Initialize(const FString& HeaderText)
{
    // Header
    auto HeaderTextBlock = NewObject<UTextBlock>(this);
    HeaderTextBlock->SetText(FText::FromString(HeaderText));
    FSlateFontInfo font = HeaderTextBlock->GetFont();
    float DPI = UWidgetLayoutLibrary::GetViewportScale(this->GetWorld()->GetGameViewport());
    font.Size = FMath::RoundToInt(20 * DPI);
    font.TypefaceFontName = "Regular";
    HeaderTextBlock->SetFont(font);
    HeaderContent = HeaderTextBlock;

    // Body
    BodyContent = NewObject<UVerticalBox>(this);
}

void UDDCharacterEditorExpandableArea::AddBoneShapingSlider(
    const FDDCharacterEditorSegmentation& Segmentation,
    const FDDTransformConstraint& TransformConstraint)
{
    // Create a new horizontal box to hold the text block and the slider
    UHorizontalBox* HorizontalBox = NewObject<UHorizontalBox>(this);

    // Create the TextBlock to display the slider name
    UTextBlock* SliderLabel = NewObject<UTextBlock>(this);
    SliderLabel->SetText(FText::FromString(TransformConstraint.displayName));
    float DPI = UWidgetLayoutLibrary::GetViewportScale(this->GetWorld()->GetGameViewport());
    FSlateFontInfo font = SliderLabel->GetFont();
    font.Size = FMath::RoundToInt(12 * DPI);
    font.TypefaceFontName = "Regular";
    SliderLabel->SetFont(font);

    // Add the TextBlock to the horizontal box
    UHorizontalBoxSlot* LabelSlot = HorizontalBox->AddChildToHorizontalBox(SliderLabel);
    LabelSlot->SetPadding(FMargin(5.0f));
    LabelSlot->SetHorizontalAlignment(HAlign_Left);
    FSlateChildSize LabelSlotSize;
    LabelSlotSize.SizeRule = ESlateSizeRule::Fill;
    LabelSlotSize.Value = 0.5f;
    LabelSlot->SetSize(LabelSlotSize);

    // Create the Slider widget
    UDDCharacterEditor1DSlider* Slider1D = CreateWidget<UDDCharacterEditor1DSlider>(GetWorld(), DDCharacterEditor1DSliderClass);
    Slider1D->Type = UDDCharacterEditor1DSlider::BoneShaping;
    Slider1D->UserData[0] = &Segmentation;
    Slider1D->UserData[1] = &TransformConstraint;
    //Slider1D->Controller = BoneShapingController;
    Slider1D->SetRange(TransformConstraint.range);

    // Add the Slider to the horizontal box
    UHorizontalBoxSlot* SliderSlot = HorizontalBox->AddChildToHorizontalBox(Slider1D);
    SliderSlot->SetPadding(FMargin(5.0f));
    SliderSlot->SetHorizontalAlignment(HAlign_Fill);
    SliderSlot->SetSize(FSlateChildSize(ESlateSizeRule::Fill));

    // Add the completed horizontal box to the body
    UVerticalBoxSlot* HorizontalBoxSlot = GetBodyAsVerticalBox()->AddChildToVerticalBox(HorizontalBox);
}

void UDDCharacterEditorExpandableArea::AddShapeConstraintsSlider(
    const FDDShapeConstraints& ShapeConstraints)
{
    // Create a new horizontal box to hold the text block and the slider
    UHorizontalBox* HorizontalBox = NewObject<UHorizontalBox>(this);

    // Create the TextBlock to display the slider name
    UTextBlock* SliderLabel = NewObject<UTextBlock>(this);
    SliderLabel->SetText(FText::FromString(ShapeConstraints.displayName));
    float DPI = UWidgetLayoutLibrary::GetViewportScale(this->GetWorld()->GetGameViewport());
    FSlateFontInfo font = SliderLabel->GetFont();
    font.Size = FMath::RoundToInt(12 * DPI);
    font.TypefaceFontName = "Regular";
    SliderLabel->SetFont(font);

    // Add the TextBlock to the horizontal box
    UHorizontalBoxSlot* LabelSlot = HorizontalBox->AddChildToHorizontalBox(SliderLabel);
    LabelSlot->SetPadding(FMargin(5.0f));
    LabelSlot->SetHorizontalAlignment(HAlign_Left);
    FSlateChildSize LabelSlotSize;
    LabelSlotSize.SizeRule = ESlateSizeRule::Fill;
    LabelSlotSize.Value = 0.5f;
    LabelSlot->SetSize(LabelSlotSize);

    UWidget* WidgetToAdd = nullptr;
    check(ShapeConstraints.shapes.Num() > 0);
    if (ShapeConstraints.shapes.Num() == 1)
    {
        // Create the 1D Slider widget
        UDDCharacterEditor1DSlider* Slider1D = CreateWidget<UDDCharacterEditor1DSlider>(GetWorld(), DDCharacterEditor1DSliderClass);
        Slider1D->Type = UDDCharacterEditor1DSlider::MorphShaping;
        FDDRange range;
        range.defaultValue = 0.0f;
        range.minValue = 0.0f;
        range.maxValue = 1.0f;
        Slider1D->SetRange(range);
        Slider1D->UserData[0] = &ShapeConstraints;
        Slider1D->Controller = MorphShapingController;
        WidgetToAdd = Slider1D;
    }
    else if (ShapeConstraints.shapes.Num() == 2)
    {
        // Create the 1D Slider widget
        UDDCharacterEditor1DSlider* Slider1D = CreateWidget<UDDCharacterEditor1DSlider>(GetWorld(), DDCharacterEditor1DSliderClass);
        Slider1D->Type = UDDCharacterEditor1DSlider::MorphShaping;
        FDDRange range;
        range.defaultValue = 0.0f;
        range.minValue = -1.0f;
        range.maxValue = 1.0f;
        Slider1D->SetRange(range);
        Slider1D->UserData[0] = &ShapeConstraints;
        Slider1D->Controller = MorphShapingController;
        WidgetToAdd = Slider1D;
    }
    else
    {
        // Create the 2D Poly slider widget
        UDDCharacterEditorPolygonSlider* SliderPoly = CreateWidget<UDDCharacterEditorPolygonSlider>(GetWorld(), DDCharacterEditorPolygonSliderClass);
        SliderPoly->SetSides(ShapeConstraints.shapes.Num());
        SliderPoly->Type = UDDCharacterEditorPolygonSlider::MorphShaping;
        SliderPoly->UserData[0] = &ShapeConstraints;
        SliderPoly->Controller = MorphShapingController;
        WidgetToAdd = SliderPoly;
    }

    // Add the Slider to the horizontal box
    UHorizontalBoxSlot* SliderSlot = HorizontalBox->AddChildToHorizontalBox(WidgetToAdd);
    SliderSlot->SetPadding(FMargin(5.0f));
    SliderSlot->SetHorizontalAlignment(HAlign_Fill);
    SliderSlot->SetSize(FSlateChildSize(ESlateSizeRule::Fill));

    // Add the completed horizontal box to the body
    UVerticalBoxSlot* HorizontalBoxSlot = GetBodyAsVerticalBox()->AddChildToVerticalBox(HorizontalBox);
}

void UDDCharacterEditorExpandableArea::AddSkinCompositingHandler(
    const FDDCharacterEditorSegmentation& Segmentation,
    const FDDCharacterEditorSkinCompositionHandler& Handler)
{
    // Create a new horizontal box to hold the text block and the slider
    UHorizontalBox* HorizontalBox = NewObject<UHorizontalBox>(this);

    // Create the TextBlock to display the slider name
    UTextBlock* SliderLabel = NewObject<UTextBlock>(this);
    SliderLabel->SetText(FText::FromString(Handler.displayName));
    float DPI = UWidgetLayoutLibrary::GetViewportScale(this->GetWorld()->GetGameViewport());
    FSlateFontInfo font = SliderLabel->GetFont();
    font.Size = FMath::RoundToInt(12 * DPI);
    font.TypefaceFontName = "Regular";
    SliderLabel->SetFont(font);

    // Add the TextBlock to the horizontal box
    UHorizontalBoxSlot* LabelSlot = HorizontalBox->AddChildToHorizontalBox(SliderLabel);
    LabelSlot->SetPadding(FMargin(5.0f));
    LabelSlot->SetHorizontalAlignment(HAlign_Left);
    FSlateChildSize LabelSlotSize;
    LabelSlotSize.SizeRule = ESlateSizeRule::Fill;
    LabelSlotSize.Value = 0.5f;
    LabelSlot->SetSize(LabelSlotSize);

    // Create the 2D Slider widget
    UUserWidget* Widget;
    if (Handler.type.Compare("PixelPickerElement") == 0)
    {
        UDDCharacterEditor2DSlider* Slider2D = CreateWidget<UDDCharacterEditor2DSlider>(GetWorld(), DDCharacterEditor2DSliderClass);
        Slider2D->DefaultValueSet = false;
        Slider2D->Type = UDDCharacterEditor2DSlider::TextureEditing;
        Slider2D->UserData[0] = &Segmentation;
        Slider2D->UserData[1] = &Handler;
        Slider2D->Controller = TextureEditingController;
        check(Handler.textureParams.Num() == 1);
        Slider2D->SetBackgroundTexture(Handler.textureParams[0]);
        Widget = Slider2D;
        CE2DSliders.Add(Slider2D);
    }
    else
    {
        throw;
    }

    // Add the widget to the horizontal box
    UHorizontalBoxSlot* SliderSlot = HorizontalBox->AddChildToHorizontalBox(Widget);
    SliderSlot->SetPadding(FMargin(5.0f));
    SliderSlot->SetHorizontalAlignment(HAlign_Fill);
    SliderSlot->SetSize(FSlateChildSize(ESlateSizeRule::Fill));

    // Add the completed horizontal box to the body
    UVerticalBoxSlot* HorizontalBoxSlot = GetBodyAsVerticalBox()->AddChildToVerticalBox(HorizontalBox);
}

UVerticalBox* UDDCharacterEditorExpandableArea::GetBodyAsVerticalBox()
{
    return reinterpret_cast<UVerticalBox*>(BodyContent.Get());
}
