
#include "Views/DDCharacterEditorSliders.h"
#include "DataAssets/DDCharacterEditorSkinCompositionData.h"
#include "Controllers/DDMorphShapingController.h"
#include "Models/DDMorphShapingModel.h"
#include "vector"


// Compute barycentric coordinates (u, v, w) for
// point p with respect to triangle (a, b, c)
extern void Barycentric(
	const FVector2f p,
	const FVector2f& a,
	const FVector2f& b,
	const FVector2f& c,
	float* u, float* v, float* w);

void UDDCharacterEditor1DSlider::OnChanged(float v)
{
	switch (Type)
	{
	case UDDCharacterEditor1DSlider::BoneShaping:
	{
		//auto BoneShapingController = reinterpret_cast<UDDBoneShapingController*>(Controller);
		//auto Segmentation = reinterpret_cast<const FDDCharacterEditorSegmentation*>(UserData[0]);
		//auto TransformConstraint = reinterpret_cast<const FDDTransformConstraint*>(UserData[1]);
		//BoneShapingController->UpdateModel(Segmentation, TransformConstraint, v);
		break;
	}
	case UDDCharacterEditor1DSlider::MorphShaping:
	{
		auto MorphShapingController = reinterpret_cast<UDDMorphShapingController*>(Controller);
		auto ShapeConstraints = reinterpret_cast<const FDDShapeConstraints*>(UserData[0]);
		if (ShapeConstraints->shapes.Num() == 1)
		{
			MorphShapingController->UpdateModel(ShapeConstraints, 0, v);
		}
		else
		{
			float v0 = std::min(0.0f, v);
			float v1 = std::max(0.0f, v);
			UE_LOG(LogTemp, Warning, TEXT("v0: %f, v1: %f"), v0, v1);
			MorphShapingController->UpdateModel(ShapeConstraints, 0, v0, 1, v1);
		}
		break;
	}
	default:
		break;
	}
}

void UDDCharacterEditor2DSlider::OnChanged(FVector2f v)
{
	switch (Type)
	{
	case UDDCharacterEditor2DSlider::MorphShaping:
	{
		//auto MorphTargetController = reinterpret_cast<UDDSegmentedShapingController*>(Controller);
		//auto Segmentation = reinterpret_cast<const FDDMorphTargetSegmentation*>(UserData[0]);
		//MorphTargetController->UpdateModel(Segmentation, v);
		break;
	}
	case UDDCharacterEditor2DSlider::TextureEditing:
	{
		auto TextureEditingController = reinterpret_cast<UDDTextureEditingController*>(Controller);
		auto Segmentation = reinterpret_cast<const FDDCharacterEditorSegmentation*>(UserData[0]);
		auto SkinCompositorHandler = reinterpret_cast<const FDDCharacterEditorSkinCompositionHandler*>(UserData[1]);
		TextureEditingController->UpdateModel(Segmentation, SkinCompositorHandler, v);
		break;
	}
	default:
		break;
	}
}

void UDDCharacterEditor2DSlider::NativeTick(const FGeometry& MyGeometry, float InDeltaTime)
{
	Super::NativeTick(MyGeometry, InDeltaTime);

	if (!DefaultValueSet)
	{
		switch (Type)
		{
		case UDDCharacterEditor2DSlider::TextureEditing:
		{
			auto Handler = static_cast<const FDDCharacterEditorSkinCompositionHandler*>(UserData[1]);
			if (Handler->floatParams.Num() == 2)
			{
				auto size = GetCachedGeometry().Size;
				auto targetHalfSize = Target->GetCachedGeometry().Size / 2.0;
				if (size.Length() > 0.0f && size.Length() > 0.0f)
				{
					// Only update to default values if the geometry has been set and we can compute
					// the pixel based position from normalized positions
					int32 x = (Handler->floatParams[0] * 0.5) * size.X ;
					int32 y = (Handler->floatParams[1] * 0.5) * size.Y + targetHalfSize.Y;
					SetPickerPosition(x, y);
					DefaultValueSet = true;
				}
			}
			else
			{
				DefaultValueSet = true;
			}
			break;
		}
		default:
			DefaultValueSet = true;
			break;
		}
	}
}

FVector2f UDDCharacterEditorPolygonSlider::AdjustValue(FVector2f Value) const
{
	//check(Vertices.Num() >= 3);
	//FGeometry WidgetGeometry = GetTickSpaceGeometry();
	//FVector2D WidgetSize = WidgetGeometry.GetLocalSize();
	//auto CenterOffset = (FVector2f)WidgetSize * 0.5f;
	//FVector2D ScaleFactor;
	//ScaleFactor.X = WidgetSize.X * 0.5f;
	//ScaleFactor.Y = WidgetSize.Y * 0.5f;
	//auto CenterVertexLocalSpace = (FVector2f)(CenterVertex * ScaleFactor) + CenterOffset;

	//FVector2f ValueInWidgetSpace = Value * (FVector2f)ScaleFactor + CenterOffset;
	//ValueInWidgetSpace.X = FMath::Clamp(ValueInWidgetSpace.X, 0.0f, (float)WidgetSize.X);
	//ValueInWidgetSpace.Y = FMath::Clamp(ValueInWidgetSpace.Y, 0.0f, (float)WidgetSize.Y);

	//float SelectedTriangleMinVal = -FLT_MAX;
	//int32 SelectedTriangleIndex = -1;
	//
	//for (int32 i = 0; i < Vertices.Num(); i++)
	//{
	//	int32 nextIndex = (i < Vertices.Num() - 1) ? i + 1 : 0;
	//	auto v0 = (FVector2f)(Vertices[i] * ScaleFactor) + CenterOffset;
	//	auto v1 = (FVector2f)(Vertices[nextIndex] * ScaleFactor) + CenterOffset;
	//	float u, v, w;
	//	Barycentric((FVector2f)ValueInWidgetSpace, CenterVertexLocalSpace, v0, v1, &u, &v, &w);
	//	std::vector<float> values = { u, v, w };
	//	float minVal = *std::min_element(values.begin(), values.end());
	//	if (SelectedTriangleMinVal < minVal)
	//	{
	//		SelectedTriangleMinVal = minVal;
	//		SelectedTriangleIndex = i;
	//	}
	//}

	//if (SelectedTriangleMinVal < 0.0f)
	//{
	//	int32 nextIndex = (SelectedTriangleIndex < Vertices.Num() - 1) ? SelectedTriangleIndex + 1 : 0;
	//	auto v0 = (FVector2f)(Vertices[SelectedTriangleIndex] * ScaleFactor) + CenterOffset;
	//	auto v1 = (FVector2f)(Vertices[nextIndex] * ScaleFactor) + CenterOffset;
	//	float u, v, w;
	//	Barycentric((FVector2f)ValueInWidgetSpace, CenterVertexLocalSpace, v0, v1, &u, &v, &w);
	//	u = std::max(0.0f, u);
	//	v = std::max(0.0f, v);
	//	w = std::max(0.0f, w);
	//	float sum = u + v + w;
	//	u = u / sum;
	//	v = v / sum;
	//	w = w / sum;
	//	ValueInWidgetSpace = CenterVertexLocalSpace * u + v0 * v + v1* w;
	//}

	//return (ValueInWidgetSpace - CenterOffset) / (FVector2f)ScaleFactor;

	return Value;
}

FVector2f UDDCharacterEditorPolygonSlider::GetSelectionPosition() const
{
	FGeometry WidgetGeometry = GetTickSpaceGeometry();
	FVector2D WidgetSize = WidgetGeometry.GetLocalSize();
	auto CenterOffset = (FVector2f)WidgetSize * 0.5f;
	FVector2D ScaleFactor;
	ScaleFactor.X = WidgetSize.X * 0.5f;
	ScaleFactor.Y = WidgetSize.Y * 0.5f;

	FVector2f ValueInWidgetSpace = (FVector2f)SelectionPosition * (FVector2f)ScaleFactor;

	return (FVector2f)SelectionPosition * (FVector2f)ScaleFactor;
}

void UDDCharacterEditorPolygonSlider::OnChanged(FVector2f Value)
{
	SelectionPosition = FVector2D(Value);

	int32 tIndex;
	float u, v, w;
	if (GetTriangleUnderValue(Value, &tIndex, &u, &v, &w))
	{
		UE_LOG(LogTemp, Warning, TEXT("i: %d, u: %f, v: %f, w: %f"), tIndex, u, v, w);

		auto MorphShapingController = reinterpret_cast<UDDMorphShapingController*>(Controller);
		auto ShapeConstraints = reinterpret_cast<const FDDShapeConstraints*>(UserData[0]);
		int32 shapeIndex_0 = tIndex;
		int32 shapeIndex_1 = (tIndex < Vertices.Num() - 1) ? tIndex + 1 : 0;
		MorphShapingController->UpdateModel(ShapeConstraints, tIndex, v, shapeIndex_1, w);
	}
}

void UDDCharacterEditorPolygonSlider::SetSides(int Sides)
{
	check(Sides >= 3); // Ensure at least a triangle
	FVector2D Center = FVector2D(0.0f, 0.0f);
	float Radius = 1.0f;
	float AngleStep = 360.0f / Sides;
	Vertices.Empty();
	FVector2D Min(TNumericLimits<float>::Max(), TNumericLimits<float>::Max());
	FVector2D Max(TNumericLimits<float>::Lowest(), TNumericLimits<float>::Lowest());

	for (int i = 0; i < Sides; i++)
	{
		// Convert degrees to radians
		float Angle = FMath::DegreesToRadians(-90.0f + i * AngleStep);

		// Calculate point position
		FVector2D Point = Center + FVector2D(
			FMath::Cos(Angle * Radius),
			FMath::Sin(Angle * Radius)
		);

		// Add to vertex array
		Vertices.Add(Point);

		// Find min and max
		Min.X = FMath::Min(Min.X, Point.X);
		Min.Y = FMath::Min(Min.Y, Point.Y);
		Max.X = FMath::Max(Max.X, Point.X);
		Max.Y = FMath::Max(Max.Y, Point.Y);
	}

	// Resize to fit perfecty in the container area
	FVector2D ShapeSize = FVector2D(Max.X - Min.X, Max.Y - Min.Y) * 0.5f;
	FVector2D ShapeCenter = FVector2D(Max.X + Min.X, Max.Y + Min.Y) * 0.5f;
	CenterVertex = -ShapeCenter;
	for (int i = 0; i < Vertices.Num(); i++)
	{
		Vertices[i] = (Vertices[i] - ShapeCenter) / ShapeSize;
	}


	SelectionPosition = CenterVertex;

}


void UDDCharacterEditorPolygonSlider::NativeConstruct()
{
	Super::NativeConstruct();


	// Reset the selected position

	FGeometry WidgetGeometry = GetTickSpaceGeometry();
	FVector2D WidgetSize = WidgetGeometry.GetLocalSize();
	FVector2D ScaleFactor;
	ScaleFactor.X = WidgetSize.X * 0.5f;
	ScaleFactor.Y = WidgetSize.Y * 0.5f;

	//SelectionPosition = CenterVertex * ScaleFactor;
}

void UDDCharacterEditorPolygonSlider::NativeTick(const FGeometry& MyGeometry, float InDeltaTime)
{
	Super::NativeTick(MyGeometry, InDeltaTime);

}

int32 UDDCharacterEditorPolygonSlider::NativePaint(
	const FPaintArgs& Args,
	const FGeometry& AllottedGeometry,
	const FSlateRect& MyCullingRect,
	FSlateWindowElementList& OutDrawElements,
	int32 LayerId,
	const FWidgetStyle& InWidgetStyle,
	bool bParentEnabled) const
{
	if (Vertices.Num() < 2)
		return LayerId; // No need to draw if there are not enough vertices

	FLinearColor ElementsColor = FLinearColor::Red;
	FVector2D WidgetSize = AllottedGeometry.GetLocalSize(); // Get the draw area's size
	FVector2D CenterOffset = WidgetSize * 0.5f; // Center the polygon in the widget

	FVector2D ScaleFactor;
	ScaleFactor.X = WidgetSize.X * 0.5f;
	ScaleFactor.Y = WidgetSize.Y * 0.5f;

	// Convert and scale vertices
	TArray<FVector2D> LinePoints;
	for (const FVector2D& Vertex : Vertices)
		LinePoints.Add(Vertex * ScaleFactor + CenterOffset);

	// Close the polygon by connecting the last point to the first
	FVector2D first = LinePoints[0];
	LinePoints.Add(first);

	// Draw the polygon outline
	FSlateDrawElement::MakeLines(
		OutDrawElements,
		LayerId,
		AllottedGeometry.ToPaintGeometry(),
		LinePoints,
		ESlateDrawEffect::None,
		FLinearColor::Red,
		true,
		LineThickness
	);

	// Lines connecting the vertices to the selection point
	for (const FVector2D& Vertex : Vertices)
	{
		LinePoints.Empty();
		LinePoints.Add(Vertex * ScaleFactor + CenterOffset);
		LinePoints.Add(SelectionPosition * ScaleFactor + CenterOffset);
		FSlateDrawElement::MakeLines(
			OutDrawElements,
			LayerId,
			AllottedGeometry.ToPaintGeometry(),
			LinePoints,
			ESlateDrawEffect::None,
			ElementsColor,
			true,
			LineThickness * 0.3f
		);
	}

	// Lines connecting the vertices to the center point
	for (const FVector2D& Vertex : Vertices)
	{
		LinePoints.Empty();
		LinePoints.Add(Vertex * ScaleFactor + CenterOffset);
		LinePoints.Add(CenterVertex * ScaleFactor + CenterOffset);
		FSlateDrawElement::MakeLines(
			OutDrawElements,
			LayerId,
			AllottedGeometry.ToPaintGeometry(),
			LinePoints,
			ESlateDrawEffect::None,
			ElementsColor,
			true,
			LineThickness * 0.3f
		);
	}

	return LayerId + 1;
}

bool UDDCharacterEditorPolygonSlider::GetTriangleUnderValue(FVector2f Value, int32* tIndex, float* tU, float* tV, float* tW)
{
	check(Vertices.Num() >= 3);
	FGeometry WidgetGeometry = GetTickSpaceGeometry();
	FVector2D WidgetSize = WidgetGeometry.GetLocalSize();
	auto CenterOffset = (FVector2f)WidgetSize * 0.5f;
	FVector2D ScaleFactor;
	ScaleFactor.X = WidgetSize.X * 0.5f;
	ScaleFactor.Y = WidgetSize.Y * 0.5f;
	auto CenterVertexLocalSpace = (FVector2f)(CenterVertex * ScaleFactor) + CenterOffset;

	FVector2f ValueInWidgetSpace = Value * (FVector2f)ScaleFactor + CenterOffset;
	ValueInWidgetSpace.X = FMath::Clamp(ValueInWidgetSpace.X, 0.0f, (float)WidgetSize.X);
	ValueInWidgetSpace.Y = FMath::Clamp(ValueInWidgetSpace.Y, 0.0f, (float)WidgetSize.Y);

	for (int32 i = 0; i < Vertices.Num(); i++)
	{
		int32 nextIndex = (i < Vertices.Num() - 1) ? i + 1 : 0;
		auto v0 = (FVector2f)(Vertices[i] * ScaleFactor) + CenterOffset;
		auto v1 = (FVector2f)(Vertices[nextIndex] * ScaleFactor) + CenterOffset;
		float u, v, w;
		Barycentric((FVector2f)ValueInWidgetSpace, CenterVertexLocalSpace, v0, v1, &u, &v, &w);
		std::vector<float> values = { u, v, w };
		float minVal = *std::min_element(values.begin(), values.end());
		if (minVal > 0.0f)
		{
			*tIndex = i;
			*tU = u;
			*tV = v;
			*tW = w;
			return true;
		}
	}
	return false;
}
