// Fill out your copyright notice in the Description page of Project Settings.

#include "Views/DDCharacterEditorWidget.h"
#include "DataAssets/DDDataAssets.h"
#include "DataAssets/DDCharacterEditorSkinCompositionData.h"


void UDDCharacterEditorWidget::NativeConstruct()
{
    Super::NativeConstruct();

    checkf(SettingsList != nullptr, TEXT("SettingsList must be initialized."));
    TMap<FString, FDDCharacterEditorSegmentation*> MeshNameToSegmentation;
    for (auto& segmentation : DDSkeletalMeshComponent->CEConfigData->segmentations)
    {
        //// Bone shaping
        //for (auto& transformConstraint : segmentation.transformConstraints)
        //{
        //    NewExpandableArea->AddBoneShapingSlider(segmentation, transformConstraint);
        //}

        // Skin composition
        for (auto& meshName : segmentation.targetSkinCompositionMeshes)
        {
            MeshNameToSegmentation.Add(meshName, &segmentation);
        }

        // Morph shaping
        for (auto& shapeConstraints : segmentation.shapeConstraints)
        {
            auto NewExpandableArea = getOrCreateExpandableArea(segmentation.displayName);
            NewExpandableArea->AddShapeConstraintsSlider(shapeConstraints);
        }
    }

    // Skin compositing
    TMap<FDDCharacterEditorSegmentation*, TSet<FString>> SegmentationToHandlerSet;
    for (auto& component : DDSkeletalMeshComponent->SkinCompositionEditorData->components)
    {
        TSet<FDDCharacterEditorSegmentation*> segmentations;
        for (auto& meshName : component.meshes)
        {
            if (MeshNameToSegmentation.Contains(meshName))
                segmentations.Add(MeshNameToSegmentation[meshName]);
        }

        if (segmentations.Num() == 0)
            continue;
        check(segmentations.Num() == 1);
        auto segmentation = *segmentations.begin();
        if (!SegmentationToHandlerSet.Contains(segmentation))
            SegmentationToHandlerSet.Add(segmentation, TSet<FString>());

        for (auto& seHandler : component.handlers)
        {
            if (SegmentationToHandlerSet[segmentation].Contains(seHandler.displayName))
                continue;
            SegmentationToHandlerSet[segmentation].Add(seHandler.displayName);
            auto NewExpandableArea = getOrCreateExpandableArea(segmentation->displayName);
            NewExpandableArea->AddSkinCompositingHandler(*segmentation, seHandler);
        }
    }
}

UDDCharacterEditorExpandableArea* UDDCharacterEditorWidget::getOrCreateExpandableArea(const FString& name)
{
    if (!ExpandableAreasMap.Contains(name))
    {
        auto NewExpandableArea = NewObject<UDDCharacterEditorExpandableArea>(this);
        NewExpandableArea->DDCharacterEditor1DSliderClass = DDCharacterEditor1DSliderClass;
        NewExpandableArea->DDCharacterEditor2DSliderClass = DDCharacterEditor2DSliderClass;
        NewExpandableArea->DDCharacterEditorPolygonSliderClass = DDCharacterEditorPolygonSliderClass;
        //NewExpandableArea->BoneShapingController = DDSkeletalMeshComponent->getBoneShapingController();
        NewExpandableArea->MorphShapingController = DDSkeletalMeshComponent->getMorphShapingController();
        NewExpandableArea->TextureEditingController = DDSkeletalMeshComponent->getTextureEditingController();
        NewExpandableArea->Initialize(name);
        SettingsList->AddChild(NewExpandableArea);
        ExpandableAreasMap.Add(name, NewExpandableArea);
    }

    return ExpandableAreasMap[name];
}
