// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "DataAssets/DDDataAssets.h"
#include "DataAssets/DDVertexCacheData.h"
#include "Components/SkeletalMeshComponent.h"
#include "Controllers/ImageProcessor/DDImageTexture.h"
#include "DDCompositeSkeletalMeshComponent.generated.h"

USTRUCT(BlueprintType)
struct FImageProcessorArray
{
	GENERATED_BODY()

	UPROPERTY()
	TArray<UDDImageProcessor*> ImageProcessors;
};

UCLASS()
class POPUL8_API UDDCompositeSkeletalMeshComponent : public USkeletalMeshComponent
{
	GENERATED_BODY()

public:
	UDDCompositeSkeletalMeshComponent();

protected:
	virtual void OnRegister() override;
	virtual void BeginPlay() override;
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

	void Bake();
	void SetTextures();

private:
	TArray<TArray<FTransform>> GetRig(const USkinnedAsset* SKAsset, const UAnimSequence* AnimSequence);
	void SetRig(USkeletalMeshComponent* SKMC, TArray<TArray<FTransform>> BoneTransformsPerFrame);
	TArray<FTransform> GetBoneTransformsForMorp(FReferenceSkeleton RefSkeleton, TArray<TArray<FTransform>> BoneTransformsPerFrame);
	void SetMorphTargets();
	void SetTextureToMaterial(UMaterialInstanceDynamic* DynamicMaterial, const FDDTextureExport& TextureExport, const int32 VariationIndex) const;
	void CompositeCharacter();
	void CompositeSkin(USkeletalMesh* SKM);
	void BuildTextureCompositionGraph(FString MeshName, TArray<UTexture2D*> Textures, TArray<float> Weights);
	void BoneVisualizer(USkeletalMeshComponent* SKMC);
	void BoneDebugger(USkeletalMeshComponent* SKMC, TArray<FTransform> Transforms);
	void BoneDebugger(USkeletalMeshComponent* SKMC);
	void ApplyEdits();

public:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
	TArray<TObjectPtr<UDDAttachmentVertexCacheData>> AttachmentCullData;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	UDDCharacterDescriptorData* CharacterDescriptorData;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	UDDAssetCatalogData* AssetCatalogData;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	int32 GroupIndex;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	FString GroupName;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	int32 CharacterIndex;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	FString CharacterName;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	USkeletalMesh* TemplateSkeletalMesh;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	TArray<USkeletalMesh*> TemplateGarmetsSkeletalMeshes;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	UAnimSequence* AnimationSequence;

	UPROPERTY()
	TMap<FString, FImageProcessorArray> MeshNameToImageProcessorArray;

private:
	UPROPERTY()
	bool IsEditing = true;

};
