// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Components/SkeletalMeshComponent.h"
#include "Rendering/SkinWeightVertexBuffer.h"
#include "DataAssets/DDVertexCacheData.h"
#include "Controllers/DDTextureEditingController.h"
#include "Controllers/DDMorphShapingController.h"
#include "Controllers/ImageProcessor/DDImageProcessor.h"
#include "Models/DDTextureEditingModel.h"
#include "Models/DDMorphShapingModel.h"
#include "DDEditableSkeletalMeshComponent.generated.h"


UENUM(BlueprintType)
enum EDDEditableSkeletalMeshComponentState : int32
{
	ESMCS_Initial			UMETA(DisplayName = "Initial"),
	ESMCS_Editing			UMETA(DisplayName = "Editing"),
	ESMCS_Finished			UMETA(DisplayName = "Finished"),
};

UCLASS()
class POPUL8_API UDDEditableSkeletalMeshComponent : public USkeletalMeshComponent
{
	GENERATED_BODY()

public:
	UDDEditableSkeletalMeshComponent();

	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void BeginEditing();

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void FinishEditing();

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void ApplyCullData(UDDAttachmentVertexCacheData* AtthVertexCacheData);

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	class UDDMorphShapingModel* getMorphShapingModel() { return MorphShapingModel; }

	class UDDMorphShapingController* getMorphShapingController() { return MorphShapingController; }

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	class UDDTextureEditingModel* getTextureEditingModel() { return TextureEditingModel; }

	class UDDTextureEditingController* getTextureEditingController() { return TextureEditingController; }

protected:
	virtual void BeginPlay() override;
	virtual void OnRegister() override;

private:
	void ApplyEdits();

public:
	UPROPERTY(BlueprintReadOnly, Category = "Didimo")
	TEnumAsByte<EDDEditableSkeletalMeshComponentState> State = EDDEditableSkeletalMeshComponentState::ESMCS_Initial;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
	UDDCharacterEditorConfigData* CEConfigData;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
	UDDCharacterEditorSkinCompositionData* SkinCompositionEditorData;

private:
	UPROPERTY()
	TObjectPtr<UDDTextureEditingModel> TextureEditingModel;

	UPROPERTY()
	TObjectPtr<UDDTextureEditingController> TextureEditingController;



	UPROPERTY()
	TObjectPtr<UDDMorphShapingModel> MorphShapingModel;

	UPROPERTY()
	TObjectPtr<UDDMorphShapingController> MorphShapingController;



	UPROPERTY()
	TObjectPtr<UDDAttachmentVertexCacheData> AttachmentVertexCacheData;

	UPROPERTY()
	UTexture2D* UserDataTexture_0;

	FUpdateTextureRegion2D UserDataTextureUpdateRegion;

	bool bCullDataMarkedDirty = true;
};
