#pragma once

#include "CoreMinimal.h"
#include "Engine/Texture.h"
#include "ImageProcessor/DDImageProcessor.h"
#include "DDTextureEditingController.generated.h"


USTRUCT(BlueprintType)
struct POPUL8_API FDDTextureEditingColorPickerData
{
    GENERATED_BODY()

public:
    UPROPERTY()
    UTexture2D* texture2D;

    UPROPERTY()
    TArray<FColor> textureColors;

    UPROPERTY()
    int64 width;

    UPROPERTY()
    int64 height;

    UPROPERTY()
    TArray<int64> components;

    UPROPERTY()
    TArray<int64> stacks;

    UPROPERTY()
    TArray<int64> processes;

    bool SetTextureColors();
    FColor SampleColor(FVector2f TexCoord) const;
};

UCLASS(Blueprintable)
class POPUL8_API UDDTextureEditingController : public UObject
{
    GENERATED_BODY()

public:
    void Initialize(class UDDCharacterEditorConfigData* CEConfig, class UDDCharacterEditorSkinCompositionData* Config);
    void SetModel(class UDDTextureEditingModel* Model);

    void ResetToConfigValues();
    void UpdateModel(
        const struct FDDCharacterEditorSegmentation* Segmentation,
        const struct FDDCharacterEditorSkinCompositionHandler* SkinCompositorHandler,
        FVector2f value);
    void ApplyChanges();

    // TODO: Make this private in the future
//private:
    static void ProcessImageGameThread(UDDImageProcessor* ResultImageProcessor);
    static void ProcessImageGameThread(TSet<UDDImageProcessor*> ResultImageProcessors);

private:
    class UDDCharacterEditorSkinCompositionData* CEConfigData;
    class UDDTextureEditingModel* ModelToControl;

    UPROPERTY()
    TArray<FDDTextureEditingColorPickerData> ColorPickers;

    TMap<FDDCharacterEditorSkinCompositionHandler*, int32> HandlerToColorPickerIndex;
};