#pragma once

#include "CoreMinimal.h"
#include "Engine/Texture.h"
#include "Engine/TextureRenderTarget2D.h"
#include "RenderGraph.h"
#include "RenderGraphBuilder.h"
#include "DDImageProcessor.generated.h"


class UDDImageProcessor;

struct POPUL8_API DDImageProcessorExecuteContext_GameThread
{
    void Execute(UDDImageProcessor* ip);
    int32 Depth;
};

struct POPUL8_API DDImageProcessorExecuteContext_RenderThread
{
    void Execute(UDDImageProcessor* ip);
    int32 Depth;
    FRDGBuilder* GraphBuilder;
};

UCLASS(Abstract)
class POPUL8_API UDDImageProcessor : public UObject
{
    GENERATED_BODY()

    friend class UDDTextureEditingController;
    friend class UDDependantImageProcessor;
    friend struct DDImageProcessorExecuteContext_GameThread;
    friend struct DDImageProcessorExecuteContext_RenderThread;

public:
    UFUNCTION(BlueprintCallable, Category = "Didimo")
    UTextureRenderTarget2D* GetResultRenderTarget() { return ResultRenderTarget; };

    UFUNCTION()
    virtual void Initialize(int32 Width, int32 Height);

protected:
    virtual void Prepare();
    virtual void Execute(DDImageProcessorExecuteContext_GameThread& context) {};
    virtual void Execute(DDImageProcessorExecuteContext_RenderThread& context) {};
    void CreateResultRenderTarget(const FLinearColor& clearColor, int32 sizeX, int32 sizeY, ETextureRenderTargetFormat format = ETextureRenderTargetFormat::RTF_RGBA8);

protected:
    UPROPERTY()
    UTextureRenderTarget2D* ResultRenderTarget;

    FCriticalSection DataMutex;

    bool Executed_GameThread;
    bool Executed_RenderThread;
};

UCLASS(Abstract)
class POPUL8_API UDDependantImageProcessor : public UDDImageProcessor
{
    GENERATED_BODY()

public:
    void SetDependency(UDDImageProcessor* Dependency, int32 Slot = 0);

protected:
    void Prepare() override;
    void ExecuteDependencies(DDImageProcessorExecuteContext_GameThread& context);
    void ExecuteDependencies(DDImageProcessorExecuteContext_RenderThread& context);

    FRDGTextureSRV* InputDependencyAsSRV(DDImageProcessorExecuteContext_RenderThread& context, int32 DependencyIndex);

protected:
    UPROPERTY()
    TMap<int32, UDDImageProcessor*> SlotToDependency;
};