// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "DDModelSerializer.generated.h"


class UDDEditableSkeletalMeshComponent;
class UDDMorphShapingModel;
class UDDTextureEditingModel;

UCLASS(BlueprintType)
class POPUL8_API UDDModelSerializer : public UObject
{
	GENERATED_BODY()

public:
	UDDModelSerializer(const FObjectInitializer& ObjectInitializer);
	~UDDModelSerializer();


	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void AssignDataModels(UDDEditableSkeletalMeshComponent* SkeletalMeshComponent);

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void SetMorphShapingModel(UDDMorphShapingModel* Model) { MorphShapingModel = Model; }

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void SetTextureEditingModel(UDDTextureEditingModel* Model) { TextureEditingModel = Model; }

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	bool SerializeToBytes(TArray<uint8>& OutBytes);

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	bool DeserializeFromBytes(const TArray<uint8>& InBytes);

	//
	// Statics
	//

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	static bool SaveToFile(const FString& FilePath, const TArray<uint8>& Bytes);

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	static bool LoadFromFile(const FString& FilePath, TArray<uint8>& OutBytes);

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	static bool SerializeEditableSkeletalMeshComponent(UDDEditableSkeletalMeshComponent* SkeletalMeshComponent, TArray<uint8>& OutBytes);
	
	UFUNCTION(BlueprintCallable, Category = "Didimo")
	static bool DeserializeEditableSkeletalMeshComponent(UDDEditableSkeletalMeshComponent* SkeletalMeshComponent, const TArray<uint8>& Bytes);

private:
	UDDMorphShapingModel* MorphShapingModel = nullptr;
	UDDTextureEditingModel* TextureEditingModel = nullptr;
};