// Fill out your copyright notice in the Description page of Project Settings.

#pragma once
#include "CoreMinimal.h"
#include "Engine/DataAsset.h"
#include "Engine/Texture2D.h"
#include "UObject/ObjectMacros.h"
#include "UObject/ScriptMacros.h"
#include "UObject/StrongObjectPtr.h"
#include "DDCharacterEditorSkinCompositionData.generated.h"


USTRUCT(BlueprintType)
struct POPUL8_API FDDCharacterEditorSkinCompositionProcessor
{
    GENERATED_BODY()

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    FString name;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    FString type;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    FString blendMode;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<int32> intParams;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<float> floatParams;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<FString> stringParams;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<TSoftObjectPtr<UTexture2D>> textureReferences;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDCharacterEditorSkinCompositionProcessorStack
{
    GENERATED_BODY()

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    FString name;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    FString materialKey;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<FDDCharacterEditorSkinCompositionProcessor> processors;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDCharacterEditorSkinCompositionHandler
{
    GENERATED_BODY()

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    FString type;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    FString displayName;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    int32 stack;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    int32 process;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    FString field;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<int32> intParams;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<float> floatParams;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<FString> stringParams;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<UTexture2D*> textureParams;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDCharacterEditorSkinCompositionComponent
{
    GENERATED_BODY()

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<FString> meshes;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<FDDCharacterEditorSkinCompositionProcessorStack> processorStacks;

    UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Didimo")
    TArray<FDDCharacterEditorSkinCompositionHandler> handlers;
};

UCLASS(BlueprintType)
class POPUL8_API UDDCharacterEditorSkinCompositionData : public UDataAsset
{
    GENERATED_BODY()

public:
    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int version;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDCharacterEditorSkinCompositionComponent> components;


public:
    static TStrongObjectPtr<UDDCharacterEditorSkinCompositionData> Load(const FString& Path);

};