// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Engine/DataAsset.h"
#include "Engine/Texture2D.h"
#include "UObject/ObjectMacros.h"
#include "UObject/ScriptMacros.h"
#include "UObject/StrongObjectPtr.h"
#include "DDDataAssets.generated.h"

//
// Asset Catalog Data
//

USTRUCT(BlueprintType)
struct POPUL8_API FDDLODData
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int lodIndex;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString fbxPath;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDTexture
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString path;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    bool isLinear;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    UTexture2D* texture;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDMesh
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString name;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString identifier;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int submeshCount;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDTextureExport
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString key;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    bool isBlendable;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString slot;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<int> textureVariationIndices;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDMaterial
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString name;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString shader;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDTextureExport> textureExports;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDSkinnedRenderObject
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 meshExportIndex;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<int32> materialExportIndex;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString meshName;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	TArray<FString> materialNames;

    // TODO: culling information

};

USTRUCT(BlueprintType)
struct POPUL8_API FDDTemplateAsset
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString assetGuid;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString assetName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString path;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDLODData> LODData;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDSkinnedRenderObject> skinnedRenderObjects;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDCharacterEditor
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString characterEditorBasePath;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString characterEditorConfigPath;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString characterEditorSegmentationPath;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString templateAssetIndex;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDDeformableAsset
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString assetName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString assetGuid;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString path;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDSkinnedRenderObject> skinnedRenderObjects;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 templateIndex;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDShapeAsset
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString assetName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString path;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDSkinnedRenderObject> skinnedRenderObjects;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 shapeIndex;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 templateIndex;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDRect
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float xMin;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float yMin;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float width;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float height;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDVariantRegion
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 textureIndex;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString textureKey;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FDDRect sourceRect;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FDDRect targetRect;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDOverlayAtlasInformation
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDVariantRegion> variantRegions;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDOverlayMaterialConnection
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 materialIndex;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDOverlayAtlasInformation> overlayAtlasInformations;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDOverlayAsset
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString assetName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDOverlayMaterialConnection> overlayMaterialConnections;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 templateIndex;
};

UCLASS(BlueprintType)
class POPUL8_API UDDAssetCatalogData : public UDataAsset
{
    GENERATED_BODY()

public:
    static TStrongObjectPtr<UDDAssetCatalogData> Load(const FString& AssetCatalogFilePath);

public:
    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDTexture> textures;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDMesh> meshes;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDMaterial> materials;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDTemplateAsset> templateAssets;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDDeformableAsset> deformableAssets;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDShapeAsset> shapeAssets;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDOverlayAsset> overlayAssets;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDCharacterEditor> characterEditors;
};


//
// Character Editor Config Data
//

USTRUCT(BlueprintType)
struct POPUL8_API FDDRange
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float defaultValue;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float minValue;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float maxValue;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float standardDeviation;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDBoneTransformation
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString boneName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    bool invert;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int transformation;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FDDRange range;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDTransformConstraint
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString displayName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float value;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FDDRange range;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDBoneTransformation> boneTransformation;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDGradient
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString gradientTexturePath;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    UTexture2D* gradientTexture;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDMeshMaskTexture
{
	GENERATED_BODY()

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	FString meshName;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	FString texturePath;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDUIWeight
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float x;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float y;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDShapeConstraints
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString displayName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FDDUIWeight uiWeight;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FString> shapes;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDCharacterEditorSegmentation
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString displayName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString name;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDMeshMaskTexture> meshMasks;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDTransformConstraint> transformConstraints;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDShapeConstraints> shapeConstraints;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FString> targetSkinCompositionMeshes;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDMeshAlbedo
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString meshName;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString texturePath;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    UTexture2D* texture;
};

UCLASS(BlueprintType)
class POPUL8_API UDDCharacterEditorConfigData : public UDataAsset
{
    GENERATED_BODY()

public:
    static TStrongObjectPtr<UDDCharacterEditorConfigData> Load(const FString& AssetCatalogFilePath);

public:
    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int version;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString rigShapePath;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString animationPath;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDMeshAlbedo> meshAlbedos;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDCharacterEditorSegmentation> segmentations;
};

//
// Character Descriptor Config Data
//
USTRUCT(BlueprintType)
struct POPUL8_API FDDDeformableAssetsIndex
{
    GENERATED_BODY()

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	FString name;

	UPROPERTY(VisibleAnywhere, BlueprintReadWrite, Category = "Didimo")
	int32 deformableAssetIndex;

	UPROPERTY(VisibleAnywhere, BlueprintReadWrite, Category = "Didimo")
	int32 variationIndex;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDShapeWeight
{
	GENERATED_BODY()

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	FString name;

    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
    int32 index;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
	float weight;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDTextureWeight
{
	GENERATED_BODY()

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	FString name;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 index;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
	float weight;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDCharacterBlend
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	int32 templateAssetIndex;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    float templateWeight;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
	TArray<FDDShapeWeight> shapeWeights;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
	TArray<FDDTextureWeight> textureWeights;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Didimo")
	TArray<FDDDeformableAssetsIndex> deformableAssetsIndices;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDOverlayAssetIndex
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString name;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 overlayAssetIndex;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    int32 variationIndex;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDCharacter
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	FString name;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    FString group;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	TArray<FDDCharacterBlend> blends;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
	TArray<FDDOverlayAssetIndex> overlayAssetsIndices;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDGroups
{
    GENERATED_BODY()

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDCharacter> characters;
};

UCLASS(BlueprintType)
class POPUL8_API UDDCharacterDescriptorData : public UDataAsset
{
    GENERATED_BODY()

public:
    static TStrongObjectPtr<UDDCharacterDescriptorData> Load(const FString& AssetCatalogFilePath);

public:
    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Didimo")
    TArray<FDDGroups> groups;
};