#pragma once

#include "CoreMinimal.h"
#include "Engine/Texture.h"
#include "Engine/SkeletalMesh.h"
#include "Engine/Texture2D.h"
#include "Engine/TextureRenderTarget2D.h"
#include "DDTextureEditingModel.generated.h"


class UDDImageProcessor;
class UDDImageTexture;

USTRUCT(BlueprintType)
struct POPUL8_API FDDTextureEditingProcessorStack
{
	GENERATED_BODY()

	UPROPERTY()
	TArray<UDDImageProcessor*> ImageProcessorNodes;

	UPROPERTY()
	TArray<UDDImageProcessor*> InterfaceProcessors;
};

USTRUCT(BlueprintType)
struct POPUL8_API FDDTextureEditingComponent
{
	GENERATED_BODY()

	UPROPERTY()
	TArray<FDDTextureEditingProcessorStack> ImageProcessorStacks;
};

UENUM(BlueprintType)
enum EDDTextureEditingMaterialKey : int32
{
	MK_Albedo		UMETA(DisplayName = "Albedo"),
	MK_Normal		UMETA(DisplayName = "Normal"),
	MK_None			UMETA(DisplayName = "None"),
};

USTRUCT(BlueprintType)
struct FDDTextureEditingBindingKey
{
    GENERATED_BODY()

public:
    FString MeshName;
	EDDTextureEditingMaterialKey MaterialKey;

	FDDTextureEditingBindingKey();
	FDDTextureEditingBindingKey(const FString& MeshNameStr, const FString& KeyMaterialKeyStr);
	FDDTextureEditingBindingKey(const FString& MeshNameStr, EDDTextureEditingMaterialKey KeyMaterialKeyStr);

	bool operator==(const FDDTextureEditingBindingKey& Other) const;
	friend uint32 GetTypeHash(const FDDTextureEditingBindingKey& Key);
};

UCLASS(Blueprintable)
class POPUL8_API UDDTextureEditingModel : public UObject
{
    GENERATED_BODY()

	friend class UDDTextureEditingController;
	friend class UDDModelSerializer;

public:
	void Initialize(
		class UDDCharacterEditorSkinCompositionData* Config,
		USkeletalMesh* SkinnedMesh);
	bool SupportsMesh(const FDDTextureEditingBindingKey& Key) const;
	UTextureRenderTarget2D* GetMeshRenderTarget(const FDDTextureEditingBindingKey& Key);
	bool IsMarkedDirty() const { return MarkedDirty; }

private:
	void SetImageTextureInputTexture(
		UDDImageTexture* ImageTextureProcessor,
		const FString& Path);
	UDDImageProcessor* AddImageProcessor(
		const FString& BlendMode,
		const TArray<FString>& stringParams,
		const TArray<int32>& intParams,
		UDDImageProcessor* PreviousProcessor);
	UDDImageProcessor* AddColorizeProcessor(
		const FString& BlendMode,
		const TArray<FString>& stringParams,
		const TArray<int32>& intParams,
		UDDImageProcessor* PreviousProcessor);
	UDDImageProcessor* AddBlendingProcessor(
		UDDImageProcessor* AProcessor,
		UDDImageProcessor* BProcessor,
		UDDImageProcessor* MaskProcessor);


	UDDImageProcessor* AddOptionalBlending(
		const FString& BlendMode,
		UDDImageProcessor* CurrentProcessor,
		UDDImageProcessor* PreviousProcessor);

private:
	bool MarkedDirty;
	FString ConfigDir;
	FDDTextureEditingComponent* CurrentComponent;
	FDDTextureEditingProcessorStack* CurrentStack;

	UPROPERTY()
	TMap<FDDTextureEditingBindingKey, UTextureRenderTarget2D*> BindingToRenderTarget;

	UPROPERTY()
	TArray<FDDTextureEditingComponent> ImageProcessorComponents;

	UPROPERTY()
	TSet<UDDImageProcessor*> ResultNodes;
};