// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "DataAssets/DDDataAssets.h"
#include "Controllers/DDTextureEditingController.h"
#include "Models/DDTextureEditingModel.h"
#include "Blueprint/UserWidget.h"
#include "Components/SizeBox.h"
#include "DDCharacterEditorSliders.generated.h"

/**
 * 
 */
UCLASS()
class POPUL8_API UDDCharacterEditor1DSlider : public UUserWidget
{
	GENERATED_BODY()

	friend class UDDCharacterEditorExpandableArea;

public:
	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void OnChanged(float v);

	UFUNCTION(BlueprintImplementableEvent, BlueprintCallable, Category = "Didimo")
	void SetRange(FDDRange range);

private:
	enum { BoneShaping, MorphShaping, TextureEditing } Type;
	void* Controller;
	const void* UserData[3];
};


/**
 *
 */
UCLASS()
class POPUL8_API UDDCharacterEditor2DSlider : public UUserWidget
{
	GENERATED_BODY()

	friend class UDDCharacterEditorExpandableArea;

public:
	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void OnChanged(FVector2f v);

	UFUNCTION(BlueprintImplementableEvent, BlueprintCallable, Category = "Didimo")
	void SetBackgroundTexture(UTexture2D* BackgroundTexture);

	UFUNCTION(BlueprintImplementableEvent, BlueprintCallable, Category = "Didimo")
	void SetPickerPosition(int32 x, int32 y);

protected:
	virtual void NativeTick(const FGeometry& MyGeometry, float InDeltaTime) override;

protected:
	UPROPERTY(BlueprintReadWrite, meta = (BindWidget), Category = "Didimo")
	USizeBox* Target;

private:
	enum { MorphShaping, TextureEditing } Type;
	void* Controller;
	const void* UserData[3];
	bool DefaultValueSet;
};


/**
 *
 */
UCLASS()
class POPUL8_API UDDCharacterEditorPolygonSlider : public UUserWidget
{
	GENERATED_BODY()

	friend class UDDCharacterEditorExpandableArea;

public:
	UFUNCTION(BlueprintCallable, Category = "Didimo")
	FVector2f AdjustValue(FVector2f v) const;

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	FVector2f GetSelectionPosition() const;

	UFUNCTION(BlueprintCallable, Category = "Didimo")
	void OnChanged(FVector2f v);

	UFUNCTION(BlueprintImplementableEvent, BlueprintCallable, Category = "Didimo")
	void SetBackgroundTexture(UTexture2D* BackgroundTexture);

	UFUNCTION(BlueprintImplementableEvent, BlueprintCallable, Category = "Didimo")
	void SetPickerPosition(int32 x, int32 y);

	void SetSides(int Sides);

protected:
	virtual void NativeConstruct() override;
	virtual void NativeTick(const FGeometry& MyGeometry, float InDeltaTime) override;
	virtual int32 NativePaint(
		const FPaintArgs& Args,
		const FGeometry& AllottedGeometry,
		const FSlateRect& MyCullingRect,
		FSlateWindowElementList& OutDrawElements,
		int32 LayerId,
		const FWidgetStyle& InWidgetStyle,
		bool bParentEnabled) const override;

private:
	bool GetTriangleUnderValue(FVector2f Value, int32* tIndex, float* u, float* v, float* w);

protected:
	UPROPERTY(BlueprintReadWrite, meta = (BindWidget), Category = "Didimo")
	USizeBox* Target;

private:
	enum { MorphShaping, TextureEditing } Type;
	void* Controller;
	const void* UserData[3];
	bool DefaultValueSet;

	FVector2D CenterVertex;
	TArray<FVector2D> Vertices;
	FVector2D SelectionPosition;
	const float LineThickness = 2.0f;
};