#include "DDBridgeProjectImport.h"
#include "DDPopul8Editor.h"
#include "AssetImportTask.h"
#include "AssetToolsModule.h"
#include "AssetRegistry/AssetRegistryModule.h"
#include "GameFramework/Character.h"
#include "Kismet2/BlueprintEditorUtils.h"
#include "Kismet2/KismetEditorUtilities.h"
#include "Materials/MaterialExpressionTextureSampleParameter2D.h"
#include "DDBridgeUtils.h"
#include "ActorComponents/DDCompositeSkeletalMeshComponent.h"


FString UDDBridgeProjectImport::BlueprintTemplatePath = TEXT("/Popul8/BP_DDCompositeCharacterTemplate");
FString UDDBridgeProjectImport::OpaqueMaterialTemplatePath = TEXT("/Popul8/MAT_DDOpaqueTemplate");
FString UDDBridgeProjectImport::TransparentMaterialTemplatePath = TEXT("/Popul8/MAT_DDTransparentTemplate");

void UDDBridgeProjectImport::Import(const FString& SourceDir)
{
	FString DestinationDir = FPaths::Combine(TEXT("/Game"), TEXT("Didimo"), TEXT("CompositeCharacters"));

	TArray<FString> FoundFiles;
	TArray<FString> FoundDirectories;
	IFileManager::Get().FindFilesRecursive(FoundFiles, *SourceDir, TEXT("*.*"), true, false);
	FString LastDirectoryName = FPaths::GetCleanFilename(SourceDir);


	TStrongObjectPtr<UDDAssetCatalogData> AssetCatalogData = nullptr;
	FString AssetCatalogDir;
	if (!LoadAssetCatalog(FoundFiles, &AssetCatalogData, AssetCatalogDir))
	{
		UE_LOG(LogTemp, Warning, TEXT("Failed to import dir: %s"), *SourceDir);
		return;
	}

	TStrongObjectPtr<UDDCharacterDescriptorData> CharacterDescriptorData = nullptr;
	FString CharacterDescriptorDir;
	if (!LoadCharacterDescriptor(FoundFiles, &CharacterDescriptorData, CharacterDescriptorDir))
	{
		UE_LOG(LogTemp, Warning, TEXT("Failed to import dir: %s"), *SourceDir);
		return;
	}

	ImportTextures(AssetCatalogData.Get(), AssetCatalogDir, SourceDir, DestinationDir, LastDirectoryName);

	ImportMaterials(AssetCatalogData.Get(), AssetCatalogDir, SourceDir, DestinationDir, LastDirectoryName);

	ImportMeshes(AssetCatalogData.Get(), SourceDir, DestinationDir, LastDirectoryName, FoundFiles);

	TStrongObjectPtr<UDDAssetCatalogData> AssetCatalogDataPtr;
	TStrongObjectPtr<UDDCharacterDescriptorData> CharacterDescriptorDataPtr;
	ImportJSONS(DestinationDir, LastDirectoryName, AssetCatalogDir, CharacterDescriptorDir, &AssetCatalogDataPtr, &CharacterDescriptorDataPtr);

	ImportCharacters(SourceDir, DestinationDir, LastDirectoryName, AssetCatalogDataPtr, CharacterDescriptorDataPtr);

}

void UDDBridgeProjectImport::ImportTextures(UDDAssetCatalogData* AssetCatalogData, FString AssetCatalogDir,
	const FString& SourceDir, const FString& DestinationDir,
	const FString& LastDirectoryName)
{
	//Textures.SetNum(AssetCatalogData->textures.Num());
	for (int32 i = 0; i < AssetCatalogData->textures.Num(); i++)
	{
		if (!AssetCatalogData->textures[i].path.IsEmpty())
		{
			FString FilePath = FPaths::Combine(AssetCatalogDir, AssetCatalogData->textures[i].path);
			FString RelativePath = FilePath.RightChop(SourceDir.Len());
			FString DestinationFilePath = FPaths::Combine(DestinationDir, LastDirectoryName, RelativePath);
			DestinationFilePath = UDDBridgeUtils::PathSanitizer(FPaths::GetPath(DestinationFilePath));
			Textures.Add(ImportTexture(FilePath, DestinationFilePath, AssetCatalogData->textures[i].isLinear, false));
		}
		else
		{
			UE_LOG(LogTemp, Warning, TEXT("Texture not found in the Popul8 Bridge import"));
		}
	}
}

void UDDBridgeProjectImport::ImportMaterials(UDDAssetCatalogData* AssetCatalogData, const FString& AssetCatalogDir,
	const FString& SourceDir, const FString& DestinationDir,
	const FString& LastDirectoryName)
{
	for (auto TemplateAsset : AssetCatalogData->templateAssets)
		ImportMaterialsFrom(AssetCatalogData, TemplateAsset.skinnedRenderObjects, Textures,
			UDDBridgeUtils::PathSanitizer(DestinationDir / LastDirectoryName / TemplateAsset.path));

	for (auto DeformableAsset : AssetCatalogData->deformableAssets)
		ImportMaterialsFrom(AssetCatalogData, DeformableAsset.skinnedRenderObjects, Textures,
			UDDBridgeUtils::PathSanitizer(DestinationDir / LastDirectoryName / DeformableAsset.path));

	//// Creating materials from the template assets
	//for (auto TemplateAsset : AssetCatalogData->templateAssets)
	//{
	//	FString MaterialDestDir = DestinationDir / LastDirectoryName / TemplateAsset.path;
	//	for (auto SkinnedRenderObject : TemplateAsset.skinnedRenderObjects)
	//	{
	//		for (const auto MaterialIndex : SkinnedRenderObject.materialExportIndex)
	//		{
	//			FDDMaterial Material = AssetCatalogData->materials[MaterialIndex];
	//			FString MaterialName = Material.name.ToLower();
	//			if (NameToMaterial.Contains(MaterialDestDir / MaterialName))
	//				continue;

	//			FString MatTemplateName = OpaqueMaterialTemplatePath;
	//			if (Material.shader.Equals(TEXT("invisible"), ESearchCase::IgnoreCase))
	//				MatTemplateName = TransparentMaterialTemplatePath;

	//			UMaterial* NewMaterial = DuplicateDDMaterial(OpaqueMaterialTemplatePath, MaterialDestDir, MaterialName);

	//			NameToMaterial.Add(MaterialDestDir / MaterialName, NewMaterial);
	//		}
	//	}
	//}

	//// Creating materials form the deformable assets
	//for (auto DeformableAsset : AssetCatalogData->deformableAssets)
	//{
	//	FString MaterialDestDir = DestinationDir / LastDirectoryName / DeformableAsset.path;
	//	for (auto SkinnedRenderObject : DeformableAsset.skinnedRenderObjects)
	//	{
	//		for (const auto MaterialIndex : SkinnedRenderObject.materialExportIndex)
	//		{
	//			FDDMaterial Material = AssetCatalogData->materials[MaterialIndex];
	//			FString MaterialName = Material.name.ToLower();
	//			if (NameToMaterial.Contains(MaterialDestDir / MaterialName))
	//				continue;

	//			FString MatTemplateName = OpaqueMaterialTemplatePath;
	//			if (Material.shader.Equals(TEXT("invisible"), ESearchCase::IgnoreCase))
	//				MatTemplateName = TransparentMaterialTemplatePath;

	//			UMaterial* NewMaterial = DuplicateDDMaterial(OpaqueMaterialTemplatePath, MaterialDestDir, MaterialName);

	//			NameToMaterial.Add(MaterialDestDir / MaterialName, NewMaterial);
	//		}
	//	}
	//}
}

void UDDBridgeProjectImport::ImportMeshes(UDDAssetCatalogData* AssetCatalogData, const FString& SourceDir,
	const FString& DestinationDir, const FString& LastDirectoryName,
	TArray<FString> FoundFiles)
{
	// Importing template assets
	for (auto& TemplateAsset : AssetCatalogData->templateAssets)
	{
		FString SourceTemplateDir = SourceDir / TemplateAsset.LODData[0].fbxPath;
		FString DestinationTemplateDir = UDDBridgeUtils::PathSanitizer(DestinationDir / LastDirectoryName / TemplateAsset.path);
		NameToMesh.Add(TemplateAsset.LODData[0].fbxPath,
			ImportFBXMesh(&TemplateAsset.skinnedRenderObjects, SourceTemplateDir, DestinationTemplateDir, true));
	}

	for (auto& DeformableAsset : AssetCatalogData->deformableAssets)
	{
		FString SourceTemplateDir = "";
		for (auto& file : FoundFiles)
			if (FPaths::GetPath(file).Equals(SourceDir / DeformableAsset.path) && file.EndsWith("fbx"))
				SourceTemplateDir = file;

		if (SourceTemplateDir.Equals(""))
		{
			UE_LOG(LogTemp, Warning, TEXT("Failed to import Deformable Mesh: %s"), *DeformableAsset.path);
			continue;
		}

		FString DestinationDeformableDir = UDDBridgeUtils::PathSanitizer(DestinationDir / LastDirectoryName / DeformableAsset.path);
		USkeleton* OverrideSkeleton = NameToMesh[AssetCatalogData->templateAssets[DeformableAsset.templateIndex].LODData[0].fbxPath]->Skeleton;
		USkeletalMesh* DeformableMesh = ImportFBXMesh(&DeformableAsset.skinnedRenderObjects, SourceTemplateDir, DestinationDeformableDir, true, OverrideSkeleton);
		NameToMesh.Add(DeformableAsset.path, DeformableMesh);

		// Build the cull data if possible
		FString VertexCullInfoFilepath = FPaths::GetPath(SourceTemplateDir) / "VertexCullInformation.bin";
		check(DeformableAsset.templateIndex >= 0);
		auto TemplateAsset = &AssetCatalogData->templateAssets[DeformableAsset.templateIndex];
		UDDAttachmentVertexCacheData* CullData = FDDPopul8EditorModule::CreateGarmentCullingCacheData(
			VertexCullInfoFilepath,
			NameToMesh[TemplateAsset->LODData[0].fbxPath],
			DestinationDeformableDir);
		DeformableMeshToCullData.Add(DeformableMesh, CullData);
	}
}

void UDDBridgeProjectImport::ImportCharacters(const FString& SourceDir, const FString& DestinationDir,
	const FString& LastDirectoryName,
	const TStrongObjectPtr<UDDAssetCatalogData> AssetCatalogData,
	const TStrongObjectPtr<UDDCharacterDescriptorData> CharacterDescriptorData)
{
	for (int i = 0; i < CharacterDescriptorData->groups.Num(); i++)
	{
		CharacterNameToCount.Empty();
		for (int j = 0; j < CharacterDescriptorData->groups[i].characters.Num(); j++)
		{
			ImportCharacter(
				CharacterDescriptorData->groups[i].characters[j],
				SourceDir,
				DestinationDir,
				LastDirectoryName,
				AssetCatalogData,
				CharacterDescriptorData, i, j);
		}
	}
}

void UDDBridgeProjectImport::ImportCharacter(
	const FDDCharacter& Character,
	const FString& SourceDir,
	const FString& DestinationDir,
	const FString& LastDirectoryName,
	const TStrongObjectPtr<UDDAssetCatalogData> AssetCatalogData,
	const TStrongObjectPtr<UDDCharacterDescriptorData> CharacterDescriptorData,
	int32 GroupIndex, int32 CharacterIndex)
{
	// Create character BP
	FString CharacterName = UDDBridgeUtils::PathSanitizer(Character.name);
	uint32* Count = CharacterNameToCount.Find(CharacterName);
	if (Count)
		(*Count)++;
	else
		CharacterNameToCount.Add(CharacterName, 0);
	FString FormattedName = FString::Printf(TEXT("%s_%04d"), *CharacterName, CharacterNameToCount[CharacterName]);
	FString CharacterDestinationDir = UDDBridgeUtils::PathSanitizer(DestinationDir / Character.group);
	FAssetToolsModule& AssetToolsModule = FModuleManager::LoadModuleChecked<FAssetToolsModule>("AssetTools");
	FDDTemplateAsset TemplateAsset = AssetCatalogData->templateAssets[Character.blends[0].templateAssetIndex];
	//FString SourceMaterialDir = DestinationDir / LastDirectoryName / TemplateAsset.path;

	UBlueprint* BPCharacter = Cast<UBlueprint>(CreateBlueprint("BP_" + FormattedName, CharacterDestinationDir, BlueprintTemplatePath));
	if (!NameToMesh.Contains(TemplateAsset.LODData[0].fbxPath))
	{
		UE_LOG(LogTemp, Warning, TEXT("Compositing character %s %s cannot be done because template was not imported"),
			*Character.name, *Character.name);
		return;
	}
	ACharacter* aCharacter = Cast<ACharacter>(BPCharacter->GeneratedClass->GetDefaultObject());
	UDDCompositeSkeletalMeshComponent* SKMC = Cast<UDDCompositeSkeletalMeshComponent>(aCharacter->GetMesh());
	USkeletalMesh* SkeletalMesh = NameToMesh[TemplateAsset.LODData[0].fbxPath];
	SKMC->SetSkeletalMesh(SkeletalMesh);
	SKMC->TemplateSkeletalMesh = SkeletalMesh;
	SKMC->CharacterDescriptorData = CharacterDescriptorData.Get();
	SKMC->AssetCatalogData = AssetCatalogData.Get();
	SKMC->GroupIndex = GroupIndex;
	SKMC->CharacterIndex = CharacterIndex;
	SKMC->AnimationSequence = Cast<UAnimSequence>(
		StaticLoadObject(UAnimSequence::StaticClass(), nullptr, 
			*(FPaths::GetPath(SkeletalMesh->GetPathName()) / (SkeletalMesh->GetName() + "_Anim"))));

	TArray<UActorComponent*> AddedDeformables;
	for (auto& DeformableAssetIndex : Character.blends[0].deformableAssetsIndices)
	{
		if (!NameToMesh.Contains(AssetCatalogData->deformableAssets[DeformableAssetIndex.deformableAssetIndex].path))
		{
			UE_LOG(LogTemp, Warning, TEXT("Compositing character %s cannot be done because deformable %s was not imported"), *Character.name, *AssetCatalogData->deformableAssets[DeformableAssetIndex.deformableAssetIndex].path);
			return;
		}

		USkeletalMesh* DeformableTemplate = NameToMesh[AssetCatalogData->deformableAssets[DeformableAssetIndex.deformableAssetIndex].path];
		USkeletalMeshComponent* DeformableMeshComponent = NewObject<USkeletalMeshComponent>(aCharacter);
		if (!DeformableMeshComponent)
		{
			UE_LOG(LogTemp, Warning, TEXT("Failed to create Deformable Mesh Component."));
			continue;
		}
		AddedDeformables.Add(DeformableMeshComponent);
		DeformableMeshComponent->SetSkeletalMesh(DeformableTemplate);
		DeformableMeshComponent->AttachToComponent(SKMC, FAttachmentTransformRules::KeepRelativeTransform);
		DeformableMeshComponent->Rename(*AssetCatalogData->deformableAssets[DeformableAssetIndex.deformableAssetIndex].assetName);
		SKMC->TemplateGarmetsSkeletalMeshes.Add(DeformableTemplate);

		// Add cull mask for the deformable if it exists
		if (DeformableMeshToCullData.Find(DeformableTemplate))
		{
			UDDAttachmentVertexCacheData* CullData = DeformableMeshToCullData[DeformableTemplate];
			SKMC->AttachmentCullData.Add(CullData);
		}
	}

	//SKMC->Bake();

	FKismetEditorUtilities::AddComponentsToBlueprint(BPCharacter, AddedDeformables);
	FKismetEditorUtilities::CompileBlueprint(BPCharacter);
	FBlueprintEditorUtils::MarkBlueprintAsStructurallyModified(BPCharacter);

	// Create Animation Bluerpint
	USkeleton* Skeleton = SkeletalMesh->GetSkeleton();
	CreateAnimationBlueprint("ABP_" + FormattedName, CharacterDestinationDir, Skeleton);
}

void UDDBridgeProjectImport::ImportJSONS(FString DestinationDir, FString LastDirectoryName, FString AssetCatalogDir, FString CharacterDescriptorDir,
	TStrongObjectPtr<UDDAssetCatalogData>* AssetCatalogDataPtr,
	TStrongObjectPtr<UDDCharacterDescriptorData>* CharacterDescriptorDataPtr)
{
	FString RootDestinationDir = UDDBridgeUtils::PathSanitizer(FPaths::Combine(DestinationDir, LastDirectoryName));
	// Import CharacterDescriptor as its data asset type
	{
		auto JsonFactory = GetFactoryForFileType(TEXT("json"));
		FAssetToolsModule& AssetToolsModule = FModuleManager::LoadModuleChecked<FAssetToolsModule>("AssetTools");
		IAssetTools& AssetTools = AssetToolsModule.Get();

		UAssetImportTask* ImportTask = NewObject<UAssetImportTask>();
		ImportTask->Filename = CharacterDescriptorDir / "CharacterDescriptor.json";
		ImportTask->Factory = JsonFactory;
		ImportTask->DestinationPath = RootDestinationDir;
		ImportTask->bAutomated = true;

		TArray<UAssetImportTask*> ImportTasks;
		ImportTasks.Add(ImportTask);
		AssetTools.ImportAssetTasks(ImportTasks);

		*CharacterDescriptorDataPtr = TStrongObjectPtr<UDDCharacterDescriptorData>(Cast<UDDCharacterDescriptorData>(ImportTask->GetObjects()[0]));
	}

	// Import AssetCatalog as its data asset type
	{
		auto JsonFactory = GetFactoryForFileType(TEXT("json"));
		FAssetToolsModule& AssetToolsModule = FModuleManager::LoadModuleChecked<FAssetToolsModule>("AssetTools");
		IAssetTools& AssetTools = AssetToolsModule.Get();

		UAssetImportTask* ImportTask = NewObject<UAssetImportTask>();
		ImportTask->Filename = AssetCatalogDir / "AssetCatalog.json";
		ImportTask->Factory = JsonFactory;
		ImportTask->DestinationPath = RootDestinationDir;
		ImportTask->bAutomated = true;

		TArray<UAssetImportTask*> ImportTasks;
		ImportTasks.Add(ImportTask);
		AssetTools.ImportAssetTasks(ImportTasks);

		*AssetCatalogDataPtr = TStrongObjectPtr<UDDAssetCatalogData>(Cast<UDDAssetCatalogData>(ImportTask->GetObjects()[0]));
	}


}
