#pragma once
#include "DataAssets/DDCharacterEditorSkinCompositionData.h"
#include "HAL/FileManager.h"
#include "DataAssets/DDDataAssets.h"
#include "DDBridgeImportBase.generated.h"

UCLASS(Abstract)
class POPUL8EDITOR_API UDDBridgeImportBase : public UObject
{
	GENERATED_BODY()

public:
	virtual void Import(const FString& SourceDir) PURE_VIRTUAL([&]);

protected:
	virtual bool LoadAssetCatalog(const TArray<FString> FoundFiles, TStrongObjectPtr<UDDAssetCatalogData>* AssetCatalogData,
	                              FString& AssetCatalogDir);
	virtual bool LoadCharacterDescriptor(const TArray<FString> FoundFiles,
										 TStrongObjectPtr<UDDCharacterDescriptorData>* CharacterDescriptorData,
	                                     FString& CharacterDescriptorDir);
	bool LoadCESkinComposition(const FString& Path, TStrongObjectPtr<UDDCharacterEditorSkinCompositionData>* CharacterEditorSkinCompositionData, 
		FString& CESkinCompositionDir);
	virtual bool LoadCEConfig(const FString& Path, TStrongObjectPtr<UDDCharacterEditorConfigData>* CharacterEditorConfigData,
	                          FString& CEConfigDir);
	virtual UTexture2D* ImportTexture(const FString& FilePath, const FString& DestinationPath, bool isLinear,
	                                  bool useCPUAccess);
	virtual USkeletalMesh* ImportFBXMesh(
		TArray<FDDSkinnedRenderObject>* SkinnedRenderObjects,
		const FString& FilePath, 
		const FString& DestinationPath, 
		bool ImportMorphTargets, 
		USkeleton* OverrideSkeleton = nullptr);
	virtual UMaterial* CreateMaterial(	const FString& MaterialName, const FString& DestinationPath,
										UTexture2D* BC_Texture, UTexture2D* N_Texture, UTexture2D* ARM_Texture,
										bool IsAlphaBlend);
	virtual UMaterial* CreateInvisibleMaterial(const FString& MaterialName, const FString& DestinationPath);
	virtual UAnimBlueprint* CreateAnimationBlueprint(const FString& Name, const FString& DestinationPath, USkeleton* TargetSkeleton);
	virtual class UMaterialExpressionTextureSampleParameter2D* CreateMaterialTextureParameter(
		UMaterial* Material, const FString& ParameterName, UTexture* Texture, EMaterialSamplerType SamplerType,
		const FIntPoint& NodePos);
	virtual class UMaterialExpressionScalarParameter* CreateMaterialScalarParameter(
		UMaterial* Material, const FString& ParameterName, const FIntPoint& NodePos);
	virtual class UMaterialExpressionVectorParameter* CreateMaterialVectorParameter(
		UMaterial* Material, const FString& ParameterName, const FIntPoint& NodePos);
	virtual class UMaterialExpressionVertexColor* CreateMaterialVertexColor(
		UMaterial* Material, const FIntPoint& NodePos);
	virtual UMaterial* DuplicateDDMaterial(const FString& OriginalAssetPath, const FString& DuplicatedFileDirectory, const FString& AssetName);
	virtual UObject* CreateBlueprint(const FString& CharacterName, const FString& DestinationPath, const FString& BlueprintTemplatePath);
	virtual UFactory* GetFactoryForFileType(const FString& FileExtension);
	virtual void ImportMaterialsFrom(UDDAssetCatalogData* AssetCatalogData,
	                                 TArray<FDDSkinnedRenderObject> SkinnedRenderObjects,
	                                 TArray<UTexture2D*> Textures, const FString& DestinationDir);
	virtual UBlueprint* DuplicateCharacterBlueprint(const FString& OriginalAssetPath,
	                                                const FString& DuplicatedFileDirectory, const FString& AssetName);

protected:
	UPROPERTY()
	TMap<FString, UMaterial*> PathToMaterial;
};
