#pragma once
#include "DDBridgeImportBase.h"
#include "UObject/StrongObjectPtr.h"
#include "DDBridgeProjectImport.generated.h"


class UDDAttachmentVertexCacheData;

UCLASS()
class POPUL8EDITOR_API UDDBridgeProjectImport : public UDDBridgeImportBase
{
	GENERATED_BODY()

private:
	UPROPERTY()
	TArray<UTexture2D*> Textures;

	UPROPERTY()
	TMap<FString, USkeletalMesh*> NameToMesh;

	UPROPERTY()
	TMap<USkeletalMesh*, UDDAttachmentVertexCacheData*> DeformableMeshToCullData;

	static FString BlueprintTemplatePath;
	static FString OpaqueMaterialTemplatePath;
	static FString TransparentMaterialTemplatePath;


public:
	virtual void Import(const FString& SourceDir) override;

private:
	void ImportTextures(
		UDDAssetCatalogData* AssetCatalogData,
		FString AssetCatalogDir,
		const FString& SourceDir,
		const FString& DestinationDir, const FString& LastDirectoryName);
	void ImportMaterials(
		UDDAssetCatalogData* AssetCatalogData,
		const FString& AssetCatalogDir,
		const FString& SourceDir,
		const FString& DestinationDir,
		const FString& LastDirectoryName);
	void ImportMeshes(
		UDDAssetCatalogData* AssetCatalogData, 
		const FString& SourceDir, 
		const FString& DestinationDir,
		const FString& LastDirectoryName, TArray<FString> FoundFiles);
	void ImportCharacters(
		const FString& SourceDir,
		const FString& DestinationDir,
		const FString& LastDirectoryName,
		const TStrongObjectPtr<UDDAssetCatalogData> AssetCatalogData,
		const TStrongObjectPtr<UDDCharacterDescriptorData> CharacterDescriptorData);
	void ImportCharacter(
		const FDDCharacter& Character,
		const FString& SourceDir,
		const FString& DestinationDir,
		const FString& LastDirectoryName,
		const TStrongObjectPtr<UDDAssetCatalogData> AssetCatalogData,
		const TStrongObjectPtr<UDDCharacterDescriptorData> CharacterDescriptorData,
		int32 GroupIndex,
		int32 CharacterIndex);
	void ImportJSONS(
		FString DestinationDir,
		FString LastDirectoryName, 
		FString AssetCatalogDir, 
		FString CharacterDescriptorDir,
		TStrongObjectPtr<UDDAssetCatalogData>* AssetCatalogDataPtr,
		TStrongObjectPtr<UDDCharacterDescriptorData>* CharacterDescriptorDataPtr);

private:
	TMap<FString, uint32> CharacterNameToCount;
};
