// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Factories/DataAssetFactory.h"
#include "DDDataAssetFactory.generated.h"

/**
 *
 */
UCLASS()
class POPUL8EDITOR_API UDDDataAssetFactory : public UDataAssetFactory
{
	GENERATED_BODY()

public:
	UDDDataAssetFactory();

	// Called when the user tries importing a file
	virtual UObject* FactoryCreateFile(
		UClass* InClass,
		UObject* InParent,
		FName InName,
		EObjectFlags Flags,
		const FString& Filename,
		const TCHAR* Parms,
		FFeedbackContext* Warn,
		bool& bOutOperationCanceled) override;

	// Called when the user wants to create a new object
	virtual UObject* FactoryCreateNew(
		UClass* InClass,
		UObject* InParent,
		FName InName,
		EObjectFlags Flags,
		UObject* Context,
		FFeedbackContext* Warn) override;

	virtual bool FactoryCanImport(const FString& Filename) override;
	virtual UClass* ResolveSupportedClass() override;
	virtual bool DoesSupportClass(UClass* Class) override;

	// ...
	virtual bool ConfigureProperties() override;

private:
	UObject* CreateAssetCatalog(
		UClass* InClass,
		UObject* InParent,
		FName InName,
		EObjectFlags Flags);

	UObject* CreateCharacterEditorConfig(
		UClass* InClass,
		UObject* InParent,
		FName InName,
		EObjectFlags Flags);

	UObject* CreateCharacterDescriptorConfig(
		UClass* InClass,
		UObject* InParent,
		FName InName,
		EObjectFlags Flags);

    UObject* CreateCharacterEditorSkinComposition(
        UClass* InClass,
        UObject* InParent,
        FName InName,
        EObjectFlags Flags);

private:
	TSharedPtr<FJsonObject> JsonObject;
};
